package com.incubator.center;

import com.incubator.common.env.Environment;
import com.incubator.common.game.AbstractService;
import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.NetServer;
import com.incubator.common.net.netty.handler.websocket.binary.WebSocketServerBinaryAdapterHandler;
import com.incubator.common.net.netty.server.NettyServer;
import com.incubator.common.thread.NamedThreadFactory;
import com.incubator.common.util.PropertiesUtil;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.handler.InitalizerHttpServerHandler;
import com.incubator.core.net.handler.InitalizerProtocolHandler;
import com.incubator.center.listener.CenterInnerListener;
import com.incubator.center.listener.HttpInnerListener;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;

import java.io.File;
import java.io.FileInputStream;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

public class GCenter extends AbstractService {

	private static final Logger logger;

	public static ScheduledThreadPoolExecutor scheduleExec = new ScheduledThreadPoolExecutor(2, new NamedThreadFactory("taskTimer", true));
	/** 运行环境 **/
	public static final Environment env = Environment.getInstance();
	/** 服务器设置目录 **/
	public static PropertiesUtil SERVER_SETTINGS;
	public static String CONFIGURATION_FILE = "config.properties";
	public static String EVENT_FILE = "event.xml";

	/** 服务器id **/
	public static int SERVER_ID = 4000;
	/** 服务器绑定IP **/
	public static String BIND_HOST = "0.0.0.0";
	/** 服务器绑定端口 **/
	public static int BIND_PORT = 4000;
	/** 绑定的http ip **/
	public static String BIND_HTTP_HOST = "0.0.0.0";
	/** 绑定的http 端口 **/
	public static int BIND_HTTP_PORT = 4001;

	public static HashMap<Integer, NetHandler> handlers = new HashMap<>();

	private static volatile GCenter instance;

	public static GCenter getInstance() {
		if (instance == null) {
			synchronized (GCenter.class) {
				if (instance == null) {
					instance = new GCenter();
				}
			}
		}
		return instance;
	}

	static {
		// 日志加载
		Log4jUtil.initLogConfig(env.getLogConfigFile());
		logger = Log4jUtil.getLogger(GCenter.class);
	}

	public static void registerHandler(Integer cmd, NetHandler handler) {
		handlers.put(cmd, handler);
	}

	public static NetHandler getHandler(Integer cmd) {
		return handlers.get(cmd);
	}

	@Override
	protected void doStart() throws Throwable {
		// 初始化Properties配置
		this.initProperties();
		// 初始化消息事件
		this.initHandler();
		// 初始化网络服务
		this.initNetService();
		// 播网关更新游戏服链接 10秒调用
		this.scheduleExec.scheduleWithFixedDelay(new UpdateServerInfos(), 0, 10, TimeUnit.SECONDS);
	}

	@Override
	protected void doClose() throws Throwable {
		logger.info("我有关闭中控服务器!");
	}

	private void initProperties() throws Exception {
		logger.info("初始化 Properties ...");
		SERVER_SETTINGS = new PropertiesUtil(env.getConfDir() + "/" + CONFIGURATION_FILE);
		SERVER_ID = Integer.parseInt(SERVER_SETTINGS.getProperty("SERVER_ID"));

		BIND_HOST = SERVER_SETTINGS.getProperty("BIND_HOST");
		BIND_PORT = Integer.parseInt(SERVER_SETTINGS.getProperty("BIND_PORT"));

		BIND_HTTP_HOST = SERVER_SETTINGS.getProperty("BIND_HTTP_HOST");
		BIND_HTTP_PORT = Integer.parseInt(SERVER_SETTINGS.getProperty("BIND_HTTP_PORT"));
	}

	/**
	 * 加载事件
	 */
	private void initHandler() {
		logger.info("初始化消息事件...");
		if (handlers != null && handlers.size() > 0) {
			handlers.clear();
		}
		File f = new File(env.getConfDir() + "/" + EVENT_FILE);
		if (f.exists() && f.isFile()) {
			try {
				FileInputStream inputStream = new FileInputStream(f);
				SAXReader saxReader = new SAXReader();
				Document document = saxReader.read(inputStream);
				Element root = document.getRootElement();
				List<?> elements = root.elements("onDate");
				for (Object element : elements) {
					Element e = (Element) element;
					int id = Integer.parseInt(e.attributeValue("id"));
					String clazz = e.attributeValue("handleClass");
					registerHandler(id, (NetHandler) Class.forName(clazz).newInstance());
				}
			} catch (Exception e) {
				e.printStackTrace();
				logger.error(e.getMessage(), e);
			}
		}
	}

	/**
	 * 初始化网络服务
	 */
	private void initNetService() {
		logger.info("初始化网络服务...");
		NetServer innerServer = new NettyServer(GCenter.BIND_HOST, GCenter.BIND_PORT,
				new WebSocketServerBinaryAdapterHandler("/ws", new InitalizerProtocolHandler(new CenterInnerListener())));
		innerServer.start();

		NetServer innerHttpServer = new NettyServer(GCenter.BIND_HTTP_HOST, GCenter.BIND_HTTP_PORT,
				new InitalizerHttpServerHandler(new HttpInnerListener()));
		innerHttpServer.start();
	}

	/** 播网关更新游戏服链接 **/
	private static class UpdateServerInfos implements Runnable {
		@Override
		public void run() {

		}
	}
}
