package com.incubator.center.handler.http;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.Connection;
import com.incubator.common.net.netty.NettyConnection;
import com.incubator.center.service.RefreshGameNumerical;
import io.netty.buffer.Unpooled;
import io.netty.channel.ChannelFutureListener;
import io.netty.handler.codec.http.*;
import io.netty.handler.codec.http.multipart.Attribute;
import io.netty.handler.codec.http.multipart.HttpPostRequestDecoder;
import io.netty.handler.codec.http.multipart.InterfaceHttpData;
import io.netty.handler.codec.http.multipart.InterfaceHttpData.HttpDataType;
import io.netty.util.CharsetUtil;
import io.netty.util.ReferenceCountUtil;
import org.slf4j.Logger;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

public class HttpRequestEvent {

	protected Logger logger = Log4jUtil.getLogger(this.getClass());

	private static HttpRequestEvent instance;

	private Connection conn;

	private HttpRequest httpRequest;

	public static HttpRequestEvent getInstance() {
		if (instance == null) {
			instance = new HttpRequestEvent();
		}
		return instance;
	}

	public void handleRequest(Connection conn, HttpRequest msg) {
		this.conn = conn;
		this.httpRequest = msg;
		Map<String, String> args = new HashMap<>();
		QueryStringDecoder decode = new QueryStringDecoder(httpRequest.uri());
		String path = decode.path();
		try {
			if (msg.method() == HttpMethod.POST) {
				if (httpRequest instanceof FullHttpRequest) {
					FullHttpRequest request = (FullHttpRequest) httpRequest;
					HttpPostRequestDecoder d = new HttpPostRequestDecoder(request);
					for (InterfaceHttpData data : d.getBodyHttpDatas()) {
						if (data.getHttpDataType() == HttpDataType.Attribute) {
							Attribute idata = (Attribute) data;
							try {
								args.put(data.getName(), idata.getValue());
							} catch (IOException e) {
								logger.error(e.getMessage(), e);
							}
						}
					}
				}
			}
			for (Entry<String, List<String>> e : decode.parameters().entrySet()) {
				args.put(e.getKey(), e.getValue().get(0));
			}
			switch (path) {
			case "/playerToken":
				response("{\"resultcode\" : \"10000\",  \"msg\": \"处理成功!\"}");
				break;
			case "/recharge":
				if (RefreshGameNumerical.getInstance().recharge(args) == 1) {
					response("{\"resultcode\" : \"10000\",  \"msg\": \"处理成功!\"}");
				} else {
					response("{\"resultcode\" : \"20000\",  \"msg\": \"处理失败!\"}");
				}
				break;
			case "/headimg":
				if (RefreshGameNumerical.getInstance().headimg(args) == 1) {
					response("{\"resultcode\" : \"10000\",  \"msg\": \"处理成功!\"}");
				} else {
					response("{\"resultcode\" : \"20000\",  \"msg\": \"处理失败!\"}");
				}
				break;
			case "/refreshSignSetup":
				if (RefreshGameNumerical.getInstance().refreshSignSetup() == 1) {
					response("{\"resultcode\" : \"10000\",  \"msg\": \"处理成功!\"}");
				} else {
					response("{\"resultcode\" : \"20000\",  \"msg\": \"处理失败!\"}");
				}
				break;
			case "/refreshVipConfig":
				if (RefreshGameNumerical.getInstance().refreshVipConfig() == 1) {
					response("{\"resultcode\" : \"10000\",  \"msg\": \"处理成功!\"}");
				} else {
					response("{\"resultcode\" : \"20000\",  \"msg\": \"处理失败!\"}");
				}
				break;
			case "/refreshOperateConfig":
				if (RefreshGameNumerical.getInstance().refreshOperateConfig() == 1) {
					response("{\"resultcode\" : \"10000\",  \"msg\": \"处理成功!\"}");
				} else {
					response("{\"resultcode\" : \"20000\",  \"msg\": \"处理失败!\"}");
				}
				break;
			case "/playerSealProhibit":
				if (RefreshGameNumerical.getInstance().sealProhibit(args) == 1) {
					response("{\"resultcode\" : \"10000\",  \"msg\": \"处理成功!\"}");
				} else {
					response("{\"resultcode\" : \"20000\",  \"msg\": \"处理失败!\"}");
				}
				break;
			case "/refreshPlayer":
				if (RefreshGameNumerical.getInstance().refreshPlayer(args) == 1) {
					response("{\"resultcode\" : \"10000\",  \"msg\": \"处理成功!\"}");
				} else {
					response("{\"resultcode\" : \"20000\",  \"msg\": \"处理失败!\"}");
				}
				break;
			case "/refreshWhitfName":
				if (RefreshGameNumerical.getInstance().refreshWhitfName() == 1) {
					response("{\"resultcode\" : \"10000\",  \"msg\": \"处理成功!\"}");
				} else {
					response("{\"resultcode\" : \"20000\",  \"msg\": \"处理失败!\"}");
				}
				break;
			default:
				response("请求不支持");
				break;
			}
		} catch (Exception e) {
			logger.error("====http调用" + path + "出错：", e);
		} finally {
			ReferenceCountUtil.release(httpRequest);
		}
	}

	private void response(String response) {
		NettyConnection c = (NettyConnection) conn;
		FullHttpResponse httpResponse = new DefaultFullHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.OK, Unpooled.copiedBuffer(response, CharsetUtil.UTF_8));
		httpResponse.headers().set(HttpHeaderNames.CONTENT_TYPE, "text/plain; charset=UTF-8");
		c.getChannel().writeAndFlush(httpResponse).addListener(ChannelFutureListener.CLOSE);
	}

}
