package com.incubator.center.listener;

import com.incubator.center.GCenter;
import com.incubator.center.base.GameController;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.core.net.handler.listener.GameServerConnectionListener;
import com.incubator.center.manager.GameManager;
import com.incubator.message.proto.CommonProto;
import io.netty.buffer.ByteBuf;
import io.netty.handler.codec.http.websocketx.BinaryWebSocketFrame;
import io.netty.handler.codec.http.websocketx.TextWebSocketFrame;
import org.msgpack.core.MessagePack;
import org.msgpack.core.MessageUnpacker;

public class CenterInnerListener extends GameServerConnectionListener {

	@Override
	public void connectionOpened(Connection connection) {
		HeartConfig heart = new HeartConfig();
		connection.setAttribute(HeartConfig, heart);
		//关闭心跳
		setHeartEnable(false);
        logger.info("链路打开 : {}", connection);
	}

	@Override
	public void connectionClosed(Connection connection) {
        try {
            GameController game = GameManager.getInstance().find(connection);
            if (game != null) {
                logger.info("游戏服登出ID : {}", game.getInfo().getServerId());
                GameManager.getInstance().remove(game.getInfo().getServerId());
            }
        } catch (Exception e) {
			logger.error("游戏服登出处理时出错", e);
        }
    }

	@Override
	public void messageArrived(Connection conn, Object msg) {
		if (msg instanceof ByteBuf) {
			ByteBuf byteBuf = (ByteBuf) msg;

			try {
				byte[] bytes = new byte[byteBuf.readableBytes()];
				byteBuf.readBytes(bytes);
				// 提取请求数据
				WSRequest request = new WSRequest(bytes);
				// 获取处理器
				NetHandler handler = GCenter.handlers.get(request.getCmd());
				if (handler == null) {
					logger.info("消息号错误... cmd={}", request.getCmd());
					conn.close();
					return;
				}
				// 处理消息
				logger.info("收到消息 : cmd={}, 请求内容 : {}", request.getCmd(), request.toJson());
				WSResponse response = new WSResponse();
				handler.onDate(conn, request, response);
				logger.info("返回消息 : cmd={}, 返回内容 : {}", request.getCmd(), response.toJson());
			} catch (Exception e) {
				logger.error("处理消息时发生错误 : ", e);
				conn.close();
			}
		}
	}

	@Override
	public boolean doIsHeartReq(Object msg) {
		try {
			ByteBuf byteBuf = (ByteBuf) msg;
			byte[] bytes = new byte[byteBuf.readableBytes()];
			byteBuf.readBytes(bytes);
			WSRequest request = new WSRequest(bytes);
			return request.getCmd() == CommonProto.Cmd.S2S_GAME_REG_VALUE;
		} catch (Exception e) {
			logger.error("心跳请求解析失败", e);
			return false;
		}
	}

	@Override
	public void doSendHeartReq(Connection connection) {
//		HeartbeatPush push = new HeartbeatPush();
//		push.setTime(System.currentTimeMillis());
//		Message msg = new Message(ProtoCommon.MsgId.S2C_Heartbeat, push.toJson());
//		msg.setCode(ProtoCommon.Code.OK_VALUE);
//		connection.writeAndFlush(new TextWebSocketFrame(msg.toJson()));
//		logger.info("center端center端:{}", msg.getMsgId());
	}

	@Override
	public void doSendHeartRsp(Connection connection) {
//		HeartbeatPush push = new HeartbeatPush();
//		push.setTime(System.currentTimeMillis());
//		Message msg = new Message(ProtoCommon.MsgId.S2C_Heartbeat, push.toJson());
//		msg.setCode(ProtoCommon.Code.OK_VALUE);
//		connection.writeAndFlush(new TextWebSocketFrame(msg.toJson()));
//		logger.info("center端发送心跳回复：{}", msg.getMsgId());
	}

}
