package com.incubator.common.env;

/**
 * 获取运行环境
 * 
 * @author johnc
 *
 */
public abstract class Environment {
	
	public enum EnvironmentType {

		/**
		 * 正常运行环境
		 */
		Run(1, RunEnvironment.class),
		/**
		 * 开发环境
		 */
		Dev_Run(2, Dev_Environment.class),
		/**
		 * webapp运行环境
		 */
		Run_WebApp(3, Run_WebAppEnvironment.class);

		private final int value;

		private Class<? extends Environment> envImpl;

		private EnvironmentType(int value, Class<? extends Environment> impl) {
			this.value = value;
			this.envImpl = impl;
		}

		public int value() {
			return this.value;
		}

		public Class<? extends Environment> getEnvImpl() {
			return envImpl;
		}

		public void setEnvImpl(Class<? extends Environment> envImpl) {
			this.envImpl = envImpl;
		}

		public static EnvironmentType valueOf(int value) {
			for (EnvironmentType gt : values()) {
				if (gt.value == value) {
					return gt;
				}
			}
			return null;
		}
		
	}

	private static Environment env;

	private String rootDir;

	public Environment() {
		super();
	}

	public static Environment getInstance() {
		synchronized (Environment.class) {
			if (env == null) {
				env = findEnv();
				if (env == null) {
					System.err.println("Not Found Environment");
				} else {
//					System.out.println(env.info());
				}
			}
		}
		return env;
	}

	public static Environment getInstance(EnvironmentType type) {
		synchronized (Environment.class) {
			if (env == null) {
				try {
					env = type.envImpl.newInstance();
					env.initEnv();
				} catch (InstantiationException e) {
					e.printStackTrace();
				} catch (IllegalAccessException e) {
					e.printStackTrace();
				}
//				System.err.println(env.info());
			}
		}
		return env;
	}

	private static Environment findEnv() {
		for (EnvironmentType et : EnvironmentType.values()) {
			try {
				Environment env = et.envImpl.newInstance(); // 反射获取方法
				if (env.initEnv()) {
					return env;
				}
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	public final String getRootDir() {
		return rootDir;
	}

	public final void setRootDir(String rootDir) {
		this.rootDir = rootDir;
	}

	/**
	 * 脚本类目录
	 * 
	 * @return
	 */
	public abstract String getJobClassDir();

	/**
	 * 获取运行环境配置文件目录
	 * 
	 * @return
	 */
	public abstract String getConfDir();

	public abstract boolean initEnv();

	public abstract boolean initEnv(String rootDir);

	public abstract String getLogConfigFile();

	public final String info() {
		return "******************************当前环境******************************" + "\n" +
				"主目录:" + getRootDir() + "\n" +
//				"编译文件目录:" + getJobClassDir() + "\n" +
				"配置文件目录:" + getConfDir() + "\n";
	}

}