package com.incubator.common.net;

import java.net.InetSocketAddress;
import java.util.concurrent.ScheduledExecutorService;

/**
 * 网络客户端
 */
public interface NetClient {
	
	/**
	 * 服务名
	 * @return
	 */
	public String getName();

	/**
	 * 设置服务名
	 * @param name
	 */
	public void setName(String name);
	
	/**
	 * 启动
	 */
	public void start();

	/**
	 * 关闭
	 */
	public void stop();

	/**
	 * 目标地址
	 * @return
	 */
	public InetSocketAddress getTarget();

	/**
	 * 
	 * @return
	 */
	public boolean isConnected();

	/**
	 * 开启断线重连
	 * 
	 * @param executor
	 *            断线重连调度服务器
	 * @param timeMills
	 *            间隔
	 */
	public void enableReconnect(ScheduledExecutorService executor, long timeMills);

	/**
	 * 禁用断线重连
	 * 
	 * @param reconnect
	 */
	public void disableReconnect();

	/**
	 * 
	 * @return
	 */
	public boolean isReconnect();

	/**
	 * 
	 * @return
	 */
	public long getReconnectTimeMills();

	/**
	 * 发送byte数组
	 * @param data
	 */
	public void sendData(byte[] data);

	/**
	 * 发送obj
	 * @param obj
	 */
	public void sendObject(Object obj);

	/**
	 * 刷新发送缓冲区
	 */
	public void flush();
	
}
