package com.incubator.common.net.netty.handler.http;

import org.slf4j.Logger;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.ConnectionListener;
import com.incubator.common.net.netty.listener.adapter.ListenerHandlerAdapter;

import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.socket.SocketChannel;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpRequestEncoder;
import io.netty.handler.codec.http.HttpResponse;
import io.netty.handler.codec.http.HttpResponseDecoder;
import io.netty.handler.ssl.SslContext;
import io.netty.handler.ssl.SslHandler;

public class HttpClientInitHandler extends ChannelInitializer<SocketChannel> {

	protected final Logger logger = Log4jUtil.getLogger(getClass());

	private ConnectionListener<HttpResponse> listener;

	private SslContext sslCtx;

	public HttpClientInitHandler(ConnectionListener<HttpResponse> listener) {
		this.listener = listener;
	}

	/**
	 * SslContextBuilder
	 * 
	 * @param listener
	 * @param sslCtx
	 */
	public HttpClientInitHandler(ConnectionListener<HttpResponse> listener, SslContext sslCtx) {
		this.listener = listener;
		this.sslCtx = sslCtx;
	}

	@Override
	protected void initChannel(SocketChannel ch) throws Exception {
		ChannelPipeline p = ch.pipeline();
		if (sslCtx != null) {
			p.addLast(new SslHandler(sslCtx.newEngine(ch.alloc())));
		}
		p.addLast(new HttpResponseDecoder());
		// 限制contentLength
		p.addLast(new HttpObjectAggregator(65536));
		p.addLast(new HttpRequestEncoder());
		// 大文件传输处理
		// p.addLast(new ChunkedWriteHandler());
		p.addLast(new ListenerHandlerAdapter<HttpResponse>(listener));
	}

}
