package com.incubator.common.net.netty.handler.http;

import org.slf4j.Logger;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.ConnectionListener;
import com.incubator.common.net.netty.listener.adapter.ListenerHandlerAdapter;

import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.channel.socket.SocketChannel;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpRequest;
import io.netty.handler.codec.http.HttpRequestDecoder;
import io.netty.handler.codec.http.HttpResponseEncoder;
import io.netty.handler.codec.http.cors.CorsConfig;
import io.netty.handler.codec.http.cors.CorsConfigBuilder;
import io.netty.handler.codec.http.cors.CorsHandler;
import io.netty.handler.ssl.SslContext;
import io.netty.handler.ssl.SslHandler;

public class HttpServerInitHandler extends ChannelInitializer<SocketChannel> {

	protected final Logger logger = Log4jUtil.getLogger(getClass());

	private ConnectionListener<HttpRequest> listener;

	private SslContext sslCtx;

	public HttpServerInitHandler(ConnectionListener<HttpRequest> listener) {
		this.listener = listener;
	}

	/**
	 * SslContextBuilder
	 * 
	 * @param listener
	 * @param sslCtx
	 */
	public HttpServerInitHandler(ConnectionListener<HttpRequest> listener, SslContext sslCtx) {
		this.listener = listener;
		this.sslCtx = sslCtx;
	}

	@Override
	protected void initChannel(SocketChannel ch) throws Exception {
		ChannelPipeline p = ch.pipeline();
		if (sslCtx != null) {
			p.addLast(new SslHandler(sslCtx.newEngine(ch.alloc())));
		}
		p.addLast(new HttpResponseEncoder());// 必须放在最前面,如果decoder途中需要回复消息,则decoder前面需要encoder
		p.addLast(new HttpRequestDecoder());
		p.addLast(new HttpObjectAggregator(65536));// 限制contentLength
		// 大文件传输处理
		// p.addLast(new ChunkedWriteHandler());
		// p.addLast(new HttpContentCompressor());
		// 跨域配置
		CorsConfig corsConfig = CorsConfigBuilder.forAnyOrigin().allowNullOrigin().allowCredentials().build();
		p.addLast(new CorsHandler(corsConfig));
		p.addLast(new ListenerHandlerAdapter<HttpRequest>(listener));
	}

}
