package com.incubator.common.net.netty.handler.websocket;

import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler.Sharable;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelPipeline;
import io.netty.handler.codec.http.HttpObjectAggregator;
import io.netty.handler.codec.http.HttpServerCodec;
import io.netty.handler.codec.http.websocketx.WebSocketServerProtocolHandler;
import io.netty.handler.codec.http.websocketx.WebSocketServerProtocolHandler.HandshakeComplete;
import io.netty.handler.stream.ChunkedWriteHandler;
import org.slf4j.Logger;
import com.incubator.common.log4j.Log4jUtil;

/**
 * websocket服务端handler适配器 通道注册时配置解码器 通道激活时再配置监听器
 */
@Sharable
public abstract class WebSocketServerInitializer extends ChannelInitializer<Channel> {

	protected Logger logger = Log4jUtil.getLogger(getClass());
	
	protected final String uri;

	public WebSocketServerInitializer(String uri) {
		this.uri = uri;
	}

	@Override
	protected final void initChannel(Channel ch) throws Exception {
		ChannelPipeline pipeline = ch.pipeline();
		pipeline.addLast(new HttpServerCodec());
		pipeline.addLast(new ChunkedWriteHandler());
		pipeline.addLast(new HttpObjectAggregator(64 * 1024));
		pipeline.addLast(new WebSocketServerProtocolHandler(uri));
		pipeline.addLast(new WebSocketConnectedServerHandler());// 连接成功监听handler
	}

	/**
	 * 用于监测WebSocketClientProtocolHandler的事件 如果发现握手成功则构建业务handler
	 * 
	 * @author Administrator
	 */
	private class WebSocketConnectedServerHandler extends ChannelInboundHandlerAdapter {

		@Override
		public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
			if (evt instanceof HandshakeComplete) {// 新版本
				HandshakeComplete hc = (HandshakeComplete) evt;
				logger.debug("excute webSocketHandComplete……,HandshakeComplete=" + hc);
				webSocketHandComplete(ctx);
				ctx.pipeline().remove(this);
				logger.debug("excuted webSocketHandComplete:" + ctx.pipeline().toMap().toString());
				return;
			}
			super.userEventTriggered(ctx, evt);
		}
		
	}
	
	/**
	 * 当握手成功后调用该抽象方法 注意此方法加入的handler需要手动触发 ctx.fireChannelActive()
	 * ctx.fireChannelRegistered()
	 * 
	 * @param ctx
	 */
	protected abstract void webSocketHandComplete(ChannelHandlerContext ctx);
	
}
