package com.incubator.common.net.netty.server;

import com.incubator.common.thread.NamedThreadFactory;

import io.netty.channel.EventLoopGroup;
import io.netty.channel.ServerChannel;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;

/**
 * 默认后台线程
 */
public class NettyServerConfig {

	protected final Class<? extends ServerChannel> channelClass;

	protected final EventLoopGroup boss;

	protected final EventLoopGroup ioWorkers;


	public NettyServerConfig(Class<? extends ServerChannel> channelClass, EventLoopGroup boss, EventLoopGroup ioworkers) {
		this.channelClass = channelClass;
		this.boss = boss;
		this.ioWorkers = ioworkers;
	}

	public NettyServerConfig() {
		this(NioServerSocketChannel.class, new NioEventLoopGroup(0, new NamedThreadFactory("Server-boss", true)), new NioEventLoopGroup(0, new NamedThreadFactory("Server-ioWorkers", true)));
	}

	public NettyServerConfig(int bossThreads, int ioThreads) {
		this(NioServerSocketChannel.class, new NioEventLoopGroup(bossThreads, new NamedThreadFactory("Server-boss", true)), new NioEventLoopGroup(ioThreads, new NamedThreadFactory("Server-ioWorkers", true)));
	}

	public Class<? extends ServerChannel> getChannelClass() {
		return channelClass;
	}

	public EventLoopGroup getBoss() {
		return boss;
	}

	public EventLoopGroup getIoworkers() {
		return ioWorkers;
	}

	/** 毁灭 **/
	public void destory() {
		if (boss != null) {
			boss.shutdownGracefully();
		}
		if (ioWorkers != null) {
			ioWorkers.shutdownGracefully();
		}
	}

}
