package com.incubator.common.net.netty.server;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.NetServer;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.socket.ServerSocketChannel;
import org.slf4j.Logger;

import java.net.InetSocketAddress;

public abstract class NettyServerFactory implements NetServer {

	private final Logger logger = Log4jUtil.getLogger(getClass());
	
	protected final InetSocketAddress local;
	
	protected ServerSocketChannel serverChannel;

	private String name = "NettyServer";
	
	public NettyServerFactory(InetSocketAddress local) {
		this.local = local;
	}
	
	@Override
	public final boolean start() {
		return bind(this.local);
	}

	@Override
	public final void stop() {
		if (this.serverChannel != null) {
			this.serverChannel.close();
		}
	}

	@Override
	public final String getName() {
		return this.name;
	}

	@Override
	public final void setName(String name) {
		if (name != null) {
			this.name = name;
		}
	}

	@Override
	public final boolean isActive() {
		return this.serverChannel != null && this.serverChannel.isActive();
	}
	
	/**
	 * 启动端口绑定
	 * 
	 * @param local
	 * @return
	 */
	protected final boolean bind(InetSocketAddress local) {
		boolean isBind = false;
		try {
//            logger.info("端口绑定中……{}", local.toString());
			ChannelFuture cf = doBind(local);
			isBind = cf.channel() != null && cf.channel().isActive();
			if (isBind) {
//                logger.info("端口绑定成功!{}", cf.channel());
				this.serverChannel = (ServerSocketChannel) cf.channel();
			} else {
                logger.info("端口绑定失败!{}", cf.channel());
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
			throw e;
		}
		return isBind;
	}
	
	/**
	 * 使用起动器进行端口绑定
	 * 
	 * @param local
	 * @return
	 */
	protected abstract ChannelFuture doBind(InetSocketAddress local);
	
	/**
	 * 获取服务端使用的起动器
	 * 
	 * @return
	 */
	public abstract ServerBootstrap getBooter();
	
	/**
	 * 获取服务端使用的IO线程池
	 * 
	 * @return
	 */
	public abstract EventLoopGroup getIoWorkers();

	/**
	 * 获取服务端使用的选择线程池
	 * 
	 * @return
	 */
	public abstract EventLoopGroup getBossWorkers();

}
