package com.incubator.common.thread;

import com.incubator.common.log4j.Log4jUtil;

import org.slf4j.Logger;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

public abstract class AbstractRunnable implements Runnable {

	protected final Logger logger = Log4jUtil.getLogger(this.getClass());

	protected ScheduledFuture<?> timeoutFuture;

	protected static ScheduledExecutorService threadPool = Executors.newSingleThreadScheduledExecutor();

	/**
	 * 关掉线程
	 */
	protected void stopTimeoutThread() {
		if (timeoutFuture != null && !timeoutFuture.isCancelled()) {
			timeoutFuture.cancel(true);
		}
	}

	/**
	 * 隔段时间继续运行
	 * 
	 * @param timeout
	 */
	protected void startTimeoutThread(int timeout) {
		this.timeoutFuture = threadPool.schedule(this, timeout, TimeUnit.SECONDS);
	}

	/**
	 * 隔段时间继续运行
	 * 
	 * @param timeout
	 */
	protected void startTimeoutMilliThread(int timeout) {
		this.timeoutFuture = threadPool.schedule(this, timeout, TimeUnit.MILLISECONDS);
	}

}
