package com.incubator.common.util;

import org.apache.commons.lang3.StringUtils;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.UnknownHostException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * 获取地址
 * @author johnc
 *
 */
public final class NetUtil {
	
	/**
	 * 获取本地MAC地址
	 * @return MAC address
	 */
	public static String getLocalMAC(){
		StringBuilder result = new StringBuilder();
		try {
			byte[] mac = NetworkInterface.getByInetAddress(InetAddress.getLocalHost()).getHardwareAddress();
			for (int i = 0; null != mac && i < mac.length; i++) {
				if (0 != i) {
					result.append("-");
				}
				String str = Integer.toHexString(mac[i] & 0xff);
				if (1 == str.length()) {
					result.append("0").append(str);
				} else {
					result.append(str);
				}
			}
		} catch(Exception e) {
			throw new RuntimeException(e);
		}
		return result.toString().toUpperCase();
	}
	
	/**
	 * 获取MAC地址
	 * 
	 * @return MAC Address
	 */
	public static String getMAC(String ip) {
		String macAddress = getMacInWindows(ip).trim();
		if (StringUtils.isEmpty(macAddress)) {
			macAddress = getMacInLinux(ip).trim();
		}
		return macAddress;
	}
	
	/**
	 * 调用CMD命令
	 * @param cmd
	 * @return
	 */
	private static String callCmd(String[] cmd) {
		StringBuilder result = new StringBuilder();
		String line = "";
		try {
			Process proc = Runtime.getRuntime().exec(cmd);
			InputStreamReader is = new InputStreamReader(proc.getInputStream());
			BufferedReader br = new BufferedReader(is);
			while ((line = br.readLine()) != null) {
				result.append(line);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result.toString();
	}

	/**
	 * 
	 * @param cmd 第一个命令
	 * @param another 第二个命令
	 * @return 第二个命令的执行结果
	 */
	private static String callCmd(String[] cmd, String[] another) {
		StringBuilder result = new StringBuilder();
		String line = "";
		try {
			Runtime rt = Runtime.getRuntime();
			Process proc = rt.exec(cmd);
			proc.waitFor(); // 已经执行完第一个命令，准备执行第二个命令
			proc = rt.exec(another);
			InputStreamReader is = new InputStreamReader(proc.getInputStream());
			BufferedReader br = new BufferedReader(is);
			while ((line = br.readLine()) != null) {
				result.append(line);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		return result.toString();
	}

	/**
	 * 
	 * @param ip 目标IP,一般在局域网内
	 * @param sourceString 命令处理的结果字符串
	 * @param macSeparator MAC分隔符号
	 * @return MAC Address，用上面的分隔符号表示
	 */
	private static String filterMacAddress(final String ip,
			final String sourceString, final String macSeparator) {
		String result = "";
		String regExp = "((([0-9,A-F,a-f]{1,2}" + macSeparator + "){1,5})[0-9,A-F,a-f]{1,2})";
		Pattern pattern = Pattern.compile(regExp);
		Matcher matcher = pattern.matcher(sourceString);
		while (matcher.find()) {
			result = matcher.group(1);
			if (sourceString.indexOf(ip) <= sourceString.lastIndexOf(matcher
					.group(1))) {
				break; // 如果有多个IP,只匹配本IP对应的Mac.
			}
		}

		return result.toUpperCase();
	}

	/**
	 * 
	 * @param ip 目标IP
	 * @return MAC Address
	 * 
	 */
	private static String getMacInWindows(final String ip) {
		String[] cmd = { "cmd", "/c", "ping " + ip };
		String[] another = { "cmd", "/c", "arp -a" };
		String cmdResult = callCmd(cmd, another);
		return filterMacAddress(ip, cmdResult, "-");
	}

	/**
	 * 
	 * @param ip 目标IP
	 * @return MAC Address
	 * 
	 */
	private static String getMacInLinux(final String ip) {
		String[] cmd = { "/bin/sh", "-c", "ping " + ip + " -c 2 && arp -a" };
		String cmdResult = callCmd(cmd);
		return filterMacAddress(ip, cmdResult, ":");
	}
	
	public static void main(String[] args) throws UnknownHostException {
		System.out.println(InetAddress.getByName("www.360.cn").getHostAddress());
		System.out.println(getMAC("192.168.1.102"));
		System.out.println(getMAC("61.135.169.121"));
	}
	
}
