package com.incubator.common.util;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Properties;

import org.slf4j.Logger;

import com.incubator.common.log4j.Log4jUtil;

/**
 * 获取Properties文件信息
 * 
 * @author johnc
 *
 */
public final class PropertiesUtil extends Properties {

	private static final long serialVersionUID = -4599023842346938325L;

	protected Logger _log = Log4jUtil.getLogger(this.getClass());

	private boolean _warn = true;

	public PropertiesUtil() {

	}

	public PropertiesUtil setLog(boolean warn) {
		_warn = warn;
		return this;
	}

	public PropertiesUtil(String name) throws IOException {
		load(new FileInputStream(Paths.get(name).toFile()));
//		load(Files.newInputStream(Paths.get(name)));
	}

	public PropertiesUtil(File file) throws IOException {
		load(new FileInputStream(file.toPath().toFile()));
//		load(Files.newInputStream(file.toPath()));
	}

	public PropertiesUtil(InputStream inStream) throws IOException {
		load(inStream);
	}

	public PropertiesUtil(Reader reader) throws IOException {
		load(reader);
	}

	public void load(String name) throws IOException {
		load(new FileInputStream(Paths.get(name).toFile()));
//		load(Files.newInputStream(Paths.get(name)));
	}

	public void load(File file) throws IOException {
		load(new FileInputStream(file.toPath().toFile()));
//		load(Files.newInputStream(file.toPath()));
	}

	@Override
	public void load(InputStream inStream) throws IOException {
		try {
			super.load(inStream);
		} finally {
			inStream.close();
		}
	}

	@Override
	public void load(Reader reader) throws IOException {
		try {
			super.load(reader);
		} finally {
			reader.close();
		}
	}

	@Override
	public String getProperty(String key) {
		String property = super.getProperty(key);
		if (property == null) {
			if (_warn)
				_log.warn("DaProperties: Missing property for key - " + key);
			return null;
		}
		return property.trim();
	}

	@Override
	public String getProperty(String key, String defaultValue) {
		String property = super.getProperty(key, defaultValue);
		if (property == null) {
			if (_warn)
				_log.warn("DaProperties: Missing defaultValue for key - " + key);
			return null;
		}
		return property.trim();
	}

}