package com.incubator.common.util;

import java.io.UnsupportedEncodingException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public final class StringUtils {

    /**
     * 判断<code>s</code>是否为空
     * 
     * @param s
     * @return
     */
    public static boolean isEmpty(String s) {
        return (s == null || s.isEmpty());
    }

    /**
     * 去掉首位空格,判断<code>s</code>是否为空
     * 
     * @param s
     * @return
     */
    public static boolean isTrimEmpty(String s) {
        return (s == null || "".equals(s.trim()));
    }
    
    public static String trimEmpty(String s) {
        return s == null ? "" : s.trim();
    }

    public static int toInt(String s) {
        if (s == null || "".equals(s.trim())) {
            return 0;
        }
        try {
            return Integer.parseInt(s.trim());
        } catch (Exception e) {}
        return 0;
    }

    public static String[] split(String dst, String regex) {
        if (dst == null || "".equals(dst))
            return new String[0];
        return dst.split(regex);
    }

    public static int[] toInts(String dst, String regex) {
        if (dst == null || "".equals(dst))
            return null;
        String[] a = dst.split(regex);
        int[] digital = new int[a.length];
        for (int i = 0; i < a.length; i++) {
            digital[i] = Integer.parseInt(a[i]);
        }
        return digital;
    }

    /**
     * 校验是否合法的用户名
     * 
     * @param username
     * @return
     */
    public static boolean validateUsername(String username) {
        Pattern pattern = Pattern.compile("[A-Za-z0-9_]{3,10}");
        Matcher matcher = pattern.matcher(username);
        return matcher.matches();
    }

    /**
     * 获取字符串字符长度
     *
     * @param value
     * @return
     */
    public static int length(String value) {
        int valueLength = 0;
        String chinese = "[\u0391-\uFFE5]";
        /* 获取字段值的长度，如果含中文字符，则每个中文字符长度为2，否则为1 */
        for (int i = 0; i < value.length(); i++) {
            /* 获取一个字符 */
            String temp = value.substring(i, i + 1);
            /* 判断是否为中文字符 */
            if (temp.matches(chinese)) {
                /* 中文字符长度为2 */
                valueLength += 2;
            } else {
                /* 其他字符长度为1 */
                valueLength += 1;
            }
        }
        return valueLength;
    }

    /**
     * 判断字符串是否合法有效(由汉字和字母组成,并且不超过长度(字节数)<code>len</code>)
     * 
     * @param str
     * @param len
     *            限制长度 字节数( 一个汉字2个字节 1个字符1个字节)
     * @return 0=字符串合法,-1=有其他字符,-2=字符串超过长度,-3=字符串太短, -4=字符串为空
     * @author LuZhiYong
     * @Date 2012-11-7
     */
    public static int isLegStr(String str, int minLen, int maxLen) {
        if(str == null) str = "";
        Pattern pattern1 = Pattern.compile("[a-zA-Z0-9 ]");
        // pattern2 = Pattern.compile("[\u4e00-\u9fa5]");//匹配中文字符
        Pattern pattern2 = Pattern.compile("[^\\x00-\\xff]");// 匹配双字节字符
        char[] strs = str.toCharArray();

        int count = 0;
        for (char s : strs) {
            Matcher matcher1 = pattern1.matcher(String.valueOf(s));
            Matcher matcher2 = pattern2.matcher(String.valueOf(s));
            if (matcher1.find()) {
                count++;
            } else if (matcher2.find()) {
                count = count + 2;
            } else {
                return -1;
            }
        }
        if (count > maxLen)
            return -2;
        if (count < minLen)
            return -3;
        return 0;
    }

    /**
     * @param s
     * @return
     * @author TanGuiyuan
     * @date 2012-11-15
     */
    public static byte[] getUTFBytes(String s) {
        try {
            if (s == null) {
                return null;
            }
            return s.getBytes("UTF-8");
        } catch (UnsupportedEncodingException e) {
            return s.getBytes();
        }
    }

    /**
     * @param s
     * @return
     * @author TanGuiyuan
     * @date 2012-11-15
     */
    public static byte[] getGBKBytes(String s) {
        try {
            return s.getBytes("GBK");
        } catch (UnsupportedEncodingException e) {
            return s.getBytes();
        }
    }

    /**
     * @param s
     * @return
     * @author TanGuiyuan
     * @date 2012-11-15
     */
    public static byte[] getBytes(String s) {
        return s.getBytes();
    }

    /**
     * 
     * @param b
     * @return
     * @author TanGuiyuan
     * @date 2012-12-5
     */
    public static String toUTFString(byte[] b) {
        try {
            return new String(b, "UTF-8");
        } catch (UnsupportedEncodingException e) {
            return new String(b);
        }
    }

    /**
     * 验证邮箱格式是否正确
     * 
     * @param email
     * @return
     * @author TanGuiyuan
     * @date 2012-11-17
     */
    public static boolean isEmail(String email) {
        String regex = "^[\\w-]+(\\.[\\w-]+)*@[\\w-]+(\\.[\\w-]+)+$";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(email);
        return matcher.matches();
    }

    /***
     * 替换字符串某个位置的字符
     * 
     * @param idx
     *            位置
     * @param str0
     *            原字符串
     * @param str1
     *            替换的字符
     * @return Demo: idx=3,str0="0123456789",str1="x" return "012x456789";
     * @author LuZhiYong
     * @Date 2013-3-19
     */
    public static String repalce(int idx, String str0, String str1) {
        return str0.substring(0, idx) + str1 + str0.substring(idx + 1);
    }

    /****
     * 统计字符串中某个字符的数量
     * 
     * @param srcStr
     * @param find
     * @return Demo srcStr="0001010", find='0' return 5;
     * @author LuZhiYong
     * @Date 2013-3-19
     */
    public static int countStr(String srcStr, char find) {
        if (srcStr == null || srcStr.isEmpty()) {
            throw new RuntimeException("srcStr is null.");
        }
        int num = 0;
        char[] chars = srcStr.toCharArray();
        for (int i = 0; i < chars.length; i++) {
            if (find == chars[i])
                num++;
        }
        return num;
    }
    
    public static String binary2Hex(byte[] bArray) {
        StringBuffer sb = new StringBuffer(bArray.length);
        String sTemp;
        for (int i = 0; i < bArray.length; i++) {
            sTemp = Integer.toHexString(0xFF & bArray[i]);
            if (sTemp.length() < 2) {
                sb.append(0);
            }
            sb.append(sTemp.toUpperCase());
        }
        return sb.toString();
    }

    public static byte[] hex2Binary(String hexString) {
        if (StringUtils.isEmpty(hexString))
            throw new IllegalArgumentException("this hexString must not be empty");

        hexString = hexString.toLowerCase();
        final byte[] byteArray = new byte[hexString.length() / 2];
        int k = 0;
        for (int i = 0; i < byteArray.length; i++) {
            byte high = (byte) (Character.digit(hexString.charAt(k), 16) & 0xff);
            byte low = (byte) (Character.digit(hexString.charAt(k + 1), 16) & 0xff);
            byteArray[i] = (byte) (high << 4 | low);
            k += 2;
        }
        return byteArray;
    }
}
