package com.incubator.core.db;

import com.incubator.common.log4j.Log4jUtil;
import org.slf4j.Logger;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import java.util.concurrent.TimeUnit;

public class ScheduleModule {

	protected static Logger logger = Log4jUtil.getLogger(ScheduleModule.class);

	private static ScheduledExecutorService __EXECUTOR__ = Executors.newSingleThreadScheduledExecutor(new PoolFactory("系统维护", true));

	/** 添加维护任务(固定延迟)  */
	public static ScheduledFuture<?> addFixedRateJob(Runnable runnable, long initialDelay, long period) {
		try {
			return __EXECUTOR__.scheduleAtFixedRate(runnable,  Math.max(1L, initialDelay), period, TimeUnit.MILLISECONDS);
		} catch (Exception e) {
			logger.error("addFixedRateJob catch - ", e);
		}
		return null;
	}
	/** 任务执行结束后再延迟 */
	public static ScheduledFuture<?> addScheduleJob(Runnable runnable, long initialDelay, long delay, TimeUnit unit){
		try {
			return __EXECUTOR__.scheduleWithFixedDelay(runnable,  Math.max(1L, initialDelay), delay, unit);
		} catch (Exception e) {
			logger.error("ScheduleModule addScheduleJob catch - ", e);
		}
		return null;
	}
	/** 延迟执行，执行一次 */
	public static ScheduledFuture<?> addDelayJob(Runnable runnable, long delay, TimeUnit unit){
		return __EXECUTOR__.schedule(runnable, delay, unit);
	}
	public static ScheduledFuture<?> addDelayJob(Runnable runnable, long delay){
		return __EXECUTOR__.schedule(runnable, delay, TimeUnit.MILLISECONDS);
	}
}