package com.incubator.core.exception;

/**
 * 业务异常基类，所有业务异常都必须继承于此异常 .
 * 
 * @author johnc
 * 
 */
public class DBException extends RuntimeException {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2560623316827874232L;

	/**
	 * 异常信息
	 */
	protected String msg;

	/**
	 * 具体异常码
	 */
	protected int code;

	/**
	 * 数据库操作,insert返回0
	 */
	public static final DBException DB_INSERT_RESULT_0 = new DBException(40001, "数据库操作,insert返回0");

	/**
	 * 数据库操作,update返回0
	 */
	public static final DBException DB_UPDATE_RESULT_0 = new DBException(40002, "数据库操作,update返回0");

	/**
	 * 数据库操作,delete返回0
	 */
	public static final DBException DB_DELETE_RESULT_0 = new DBException(40003, "数据库操作,delete返回0");

	/**
	 * 数据库操作,selectOne返回null
	 */
	public static final DBException DB_SELECTONE_IS_NULL = new DBException(40004, "数据库操作,selectOne返回null");

	/**
	 * 数据库操作,list返回null
	 */
	public static final DBException DB_LIST_IS_NULL = new DBException(40005, "数据库操作,list返回null");

	/**
	 * Token 验证不通过
	 */
	public static final DBException TOKEN_IS_ILLICIT = new DBException(40006, "Token 验证非法");

	/**
	 * 会话超时 获取session时，如果是空，throws 下面这个异常 拦截器会拦截爆会话超时页面
	 */
	public static final DBException SESSION_IS_OUT_TIME = new DBException(40007, "会话超时");

	public DBException() {
		super();
	}

	public DBException(int code, String msgFormat, Object... args) {
		super(String.format(msgFormat, args));
		this.code = code;
		this.msg = String.format(msgFormat, args);
	}

	public DBException(String message, Throwable cause) {
		super(message, cause);
	}

	public DBException(Throwable cause) {
		super(cause);
	}

	public DBException(String message) {
		super(message);
	}

	public DBException newInstance(String msgFormat, Object... args) {
		return new DBException(this.code, msgFormat, args);
	}

	public String getMsg() {
		return msg;
	}

	public int getCode() {
		return code;
	}

}
