package com.incubator.core.net.handler.listener;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.Connection;
import com.incubator.common.net.ConnectionFactory;
import com.incubator.common.net.netty.NettyConnectionFactory;
import com.incubator.common.net.netty.listener.ConnectionHeartbeatListenerFactory;
import org.slf4j.Logger;

/**
 * 客户端链接监听
 */
public abstract class GameClientConnectionListener extends ConnectionHeartbeatListenerFactory<Object> {

	protected final Logger logger = Log4jUtil.getLogger(getClass());

	@Override
	protected boolean isHeartReq(Object msg) {
		return doIsHeartReq(msg);

//		if (msg instanceof TextWebSocketFrame) {
//			TextWebSocketFrame _msg = (TextWebSocketFrame)msg;
//			return doIsHeartReq(_msg);
//		}
//		return false;
	}

	@Override
	protected void sendHeartReq(Connection connection) {
		doSendHeartReq(connection);
	}

	@Override
	protected void sendHeartRsp(Connection connection) {
		doSendHeartRsp(connection);
	}

	@Override
	protected void doMessageArrived(Connection conn, Object msg) {
		messageArrived(conn, msg);

	}

	@Override
	public final ConnectionFactory getConnectionFactory() {
		return NettyConnectionFactory.getInstance();
	}

	/**
	 * 请求到达
	 */
	public abstract void messageArrived(Connection conn, Object msg);
	
	/**
	 * 是否是心跳请求
	 * @param msg
	 * @return
	 */
	public abstract boolean doIsHeartReq(Object msg);

	/**
	 * 发送心跳请求
	 * @param connection
	 */
	public abstract void doSendHeartReq(Connection connection);

	/**
	 * 发送心跳回复
	 * @param connection
	 */
	public abstract void doSendHeartRsp(Connection connection);
}
