package com.incubator.core.util;

import io.netty.util.AttributeKey;

import java.io.File;
import java.nio.ByteOrder;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;

/**
 * 功能描述：游戏中的常量定义
 * @author agui
 */
public interface GConst {	
	/** 秒 */
	final static int 	TIME_SECOND					= 1000;
	
	/** 分 */
	final static int 	TIME_MINUTE_SECOND 			= 60;
	final static long 	TIME_MINUTE					= TIME_MINUTE_SECOND * 1000L;
	
	/** 时 */
	final static int 	TIME_HOUR_SECOND 			= TIME_MINUTE_SECOND * 60;
	final static long 	TIME_HOUR					= TIME_HOUR_SECOND * 1000L;
	
	/** 天 */
	final static int 	TIME_DAY_SECOND 			= TIME_HOUR_SECOND * 24;
	final static long 	TIME_DAY 					= TIME_DAY_SECOND * 1000L;
	
	/** 周 */
	final static int 	TIME_WEEK_SECOND 			= TIME_DAY_SECOND * 7;
	final static long 	TIME_WEEK 					= TIME_WEEK_SECOND * 1000L;
	
	/** 月 */
	final static int 	TIME_MONTH_SECOND 			= TIME_DAY_SECOND * 30;
	final static long 	TIME_MONTH 					= TIME_MONTH_SECOND * 1000L;

	/** 年*/
	final static int 	TIME_YEAR_SECOND 			= TIME_DAY_SECOND * 365;
	final static long 	TIME_YEAR 					= TIME_YEAR_SECOND * 1000L;
	

	final static Charset UTF_8 						= StandardCharsets.UTF_8;
	
	public static final int HEAD_SIZE = 8;
	
	/** 游戏服登录密码长度 */
	public static final byte PLAT_PWD_LEN = 32;
	
	/** 手机验证码长度 */
	public static final byte PLAT_CODE_LEN	= 6;
	
	//服务器ID位数
	public static final int SERVER_ID_MAX = 100000;
	
	//全局百分比系数
	public static final int PRECNET_BASE = 10000;
	
	//全局百分比系数
	public static final float PRECNET_BASE_EXT = 0.0001f;
	
	/** 默认服务器账号上限 */
	final static int 	DEF_SERVER_ACCOUNT = 120;

	/** 允许服务器短时间内，账号进入浮动数 **/
	public static final int ACCOUNT_UP_NUM = 10;
	
	/** 默认上行字节缓存容量 */
	final static int __BUFFER_CAPACITY = 1024;//GConfig.getInstance().getInt("tcp.buffer.capacity", 256);
	
	/** 默认下行字节缓存容量 */
	final static int __BODY_CAPACITY = 1024;// GConfig.getInstance().getInt("tcp.body.capacity", 512);
	
	/** 通讯协议默认字节序 */
	final static ByteOrder __BYTE_ORDER = ByteOrder.LITTLE_ENDIAN;
	
	/** 返回包最大包体长度 */
	final static int __RESPONSE_MAX_LEN = 1024 * 512;//GConfig.getInstance().getInt("tcp.response.maxlen", 1024 * 512);
	
	/** 请求包最大包体长度 */
	final static int __REQUEST_MAX_LEN = 1024 * 10;//GConfig.getInstance().getInt("tcp.request.maxlen", 1024 * 10);


	/** 玩家标识（存放角色ID - int） */
	final static String __KEY_PLAYER = "__KEY_PLAYER";

	/** 连接中的角色账号UID */
	final static AttributeKey<String> __KEY_USER_ID = AttributeKey.valueOf("__KEY_USER_ID");

	/** 逻辑服务器ID */
	final static AttributeKey<Integer> __KEY_LOGIC_SERVERID = AttributeKey.valueOf("__KEY_LOGIC_SERVERID");

	/** 连接超时 */
	final static AttributeKey<Boolean> __KEY_SESSION_TIMEOUT = AttributeKey.valueOf("__KEY_SESSION_TIMEOUT");

	/** 登录Token */
//	final static AttributeKey<TokenInfo> __KEY_TOKEN_INFO = AttributeKey.valueOf("__KEY_TOKEN_INFO");
	
	/** 连接中的渠道 */
	final static AttributeKey<Integer> _KEY_LINK_CHANNEL = AttributeKey.valueOf("_KEY_LINK_CHANNEL");

	/** 远程回调 */
	final static byte __HEAD_CALLBACK_REMOTE = Byte.MAX_VALUE;

	/** 默认的远程回调 */
	final static byte __HEAD_CALLBACK_DEFAULT = -1;
	
	/** 目录结构 */
	final static String _ROOT_DIR_ = System.getProperty("user.dir") + File.separator + "%s" + File.separator;
	
	/** 游戏世界服务私有配置文件目录 */
	final static String DIR_CONF_SERVER = String.format(_ROOT_DIR_, "conf");
	
	/** 数据库服务配置文件名 */
	final static String FILE_CONF_DS = DIR_CONF_SERVER + "conf.ds.xml";

	/** 默认监控时长,2分钟 */
	long DEFAULT_MONITOR_TIME = 2 * 60 * 1000;
}
