package com.incubator.game;

import com.incubator.common.game.IService.ServiceState;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * 游戏服务器启动入口
 */
public class GameServerStart {

	private static final Logger logger = LoggerFactory.getLogger(GameServerStart.class);

	/**
	 * 可传两参数：
	 * 配置文件名称 项目主目录 (如：config.properties /home/work/game-center)
	 *
	 * @param args 命令行参数
	 */
	public static void main(String[] args) {
		logger.info("正在启动游戏服务器...");
		try {
			// 参数解析与初始化
			initializeConfig(args);

			// 获取游戏主实例
			GGame main = GGame.getInstance();

			// 启动服务
			main.startService();

			// 检查服务状态
			if (main.getState() != ServiceState.ACTIVE) {
				logger.error("游戏服务器启动失败，服务状态: {}", main.getState());
				System.exit(1);
			}

			logger.info("游戏服务器启动成功，服务状态: {}", main.getState());
		} catch (Exception e) {
			logger.error("游戏服务器启动过程中发生错误: ", e);
			System.exit(1);
		}
	}

	/**
	 * 初始化配置文件和环境变量
	 *
	 * @param args 命令行参数
	 * @throws IllegalArgumentException 参数异常
	 */
	private static void initializeConfig(String[] args) {
		if (args.length > 0) {
			GGame.CONFIG_FILE = args[0];
			logger.info("加载配置文件: {}", GGame.CONFIG_FILE);

			if (args.length > 1) {
				String envPath = args[1];
				logger.info("初始化环境路径: {}", envPath);
				GGame.env.initEnv(envPath);
			} else {
				logger.warn("未指定环境路径，将使用默认路径。");
			}
		} else {
			logger.warn("未指定配置文件和环境路径，使用默认配置。");
		}
	}
}
