package com.incubator.game.data.genconfig;

import com.alibaba.fastjson2.JSONArray;
import com.incubator.core.db.pool.DBFactory;
import com.incubator.core.util.FileUtil;
import com.incubator.game.GGame;
import org.apache.commons.lang3.StringUtils;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;

/**
 * 读取 json/GlobalConfig/Parameters.json
 * 自动生成 GlobalConfig 常量参数, 不需要手动复制然后覆盖参数
 *
 * @author agui 
 */
public class ParamMain {

	// java类生成头
	private final static String START = "//================================ParamMain auto generate start================================//";
	public final static String END = "//================================ParamMain auto generate end================================//";

	public static void main(String[] args) {
		loadConfig(true);
		System.exit(0);
	}
	
	public static void loadConfig(boolean loadGameConfig) {
		try {
			if (loadGameConfig) {
				GGame.getInstance().initProperties();
			}
			// Parameters.json
			initParameters();
			System.out.println("全局配置加载完成");

		} catch (Exception e) {
			System.err.println("全局配置加载出错..." + e.getMessage());
		}
	}

	private static void initParameters() {
		File parameters = new File(GGame.DIR_GAME_DATA + "/GlobalConfig/Parameters.json");
		JSONArray doc = JSONArray.parseArray(FileUtil.readText(parameters));
		StringBuilder builder = new StringBuilder(START + "\n");
		for (int i = 3; i < doc.size(); i++) {
			JSONArray values = (JSONArray) doc.get(i);
			String type = values.getString(1);
			String content = values.getString(2);
			builder.append("\n\t/**").append(content).append(" */").append("\n\tpublic static ").append("NUMBER".equals(type) ? "int " : "LONG".equals(type) ? "long " : "FLOAT".equals(type) ? "float " : "String ").append(values.get(0).toString().replace(".", "_")).append(";");
		}

		builder.append("\n\n\t" + END);

		String pathName = "src/main/java/com/incubator/game/data/config/ParametersConfig.java";
		writeToJava(pathName, builder);
	}

	private static void writeToJava(String pathName, StringBuilder builder) {
		File msgFile = new File(pathName);
		System.out.println(msgFile.getAbsolutePath());
		StringBuilder msgContent = new StringBuilder(FileUtil.readText(msgFile));
		int start = msgContent.indexOf(START);
		int end = msgContent.indexOf(END) + END.length();
		msgContent.replace(Math.max(start, 0), end, builder.toString());
		//System.out.println(msgContent.toString());
		try {
			FileWriter newMsgFile = new FileWriter(msgFile);
			newMsgFile.write(msgContent.toString());
			newMsgFile.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		System.out.println(pathName + " read init success");
	}
}