package com.incubator.game.data.jedis;

import com.incubator.game.GGame;
import org.slf4j.Logger;
import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.util.PropertiesUtil;

import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

public final class JedisPoolUtil {

	protected static Logger logger = Log4jUtil.getLogger(JedisPoolUtil.class);

	private static JedisPool jedisPool;

	private static String ADDR_ARRAY;

	private static int PORT;

	private static int TIMEOUT;

	static {
		if (jedisPool == null) {
			try {
				initPool();
			} catch (Exception e) {
				logger.error("初始化Jedis连接池失败...", e);
			}
		}
	}

	/**
	 * 初始化连接池
	 */
	public synchronized static void initPool() {
		JedisPoolConfig config = new JedisPoolConfig();
		PropertiesUtil dbredis = GGame.DB_SETTINGS;
		if (dbredis.getProperty("redis.maxTotal") != null) {
			int maxTotal = Integer.parseInt(dbredis.getProperty("redis.maxTotal"));
			config.setMaxTotal(maxTotal);
		}
		if (dbredis.getProperty("redis.maxIdle") != null) {
			int maxIdle = Integer.parseInt(dbredis.getProperty("redis.maxIdle"));
			config.setMaxIdle(maxIdle);
		}
		if (dbredis.getProperty("redis.maxWaitMillis") != null) {
			int maxWaitMillis = Integer.parseInt(dbredis.getProperty("redis.maxWaitMillis"));
			config.setMaxWaitMillis(maxWaitMillis);
		}
		if (dbredis.getProperty("redis.testOnBorrow") != null) {
			boolean testOnBorrow = Boolean.parseBoolean(dbredis.getProperty("redis.testOnBorrow"));
			config.setTestOnBorrow(testOnBorrow);
		}
		ADDR_ARRAY = dbredis.getProperty("redis.host");
		PORT = Integer.parseInt(dbredis.getProperty("redis.port"));
		TIMEOUT = Integer.parseInt(dbredis.getProperty("redis.timeout"));
		if (dbredis.getProperty("redis.auth") != null) {
			String AUTH = dbredis.getProperty("redis.auth");
			jedisPool = new JedisPool(config, ADDR_ARRAY, PORT, TIMEOUT, AUTH);
		} else {
			jedisPool = new JedisPool(config, ADDR_ARRAY, PORT, TIMEOUT);
		}
	}

	public synchronized static Jedis getJedis() {
		return jedisPool.getResource();
	}

	public static void closeJedis(Jedis jedis) {
		if (jedis != null) {
			jedis.close();
		}
	}

}
