package com.incubator.game.handler;

import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomManager;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

/**
 * 创建房间请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.CreateRoomReq_VALUE)
public class CreatRoomHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.CreateRoomRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			logger.info("创建房间失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}

		// 判断参数
		int type = request.getDataValue("type", Integer.class, 0);
		if (type <= 0) {
			logger.info("参数错误");
			response.setCode(CommonProto.Code.PARAMETER_ERR_VALUE);
			response.setMessage("参数错误");
			return;
		}

		// 玩家是否存在房间
		Room room = RoomManager.getInstance().getRoomByPlayerId(player.getId());
		if (room != null) {
			logger.info("玩家已在房间，不可创建房间");
			response.setCode(CommonProto.Code.PLAYER_ROOM_EXIST_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}

		// 创建房间
		room = RoomManager.getInstance().creatRoom(player, type);

		if (room == null) {
			logger.info("系统错误");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败");
			return;
		}

		// 玩家绑定房间
		player.data.roomId = room.data.roomId;

		// 开启房间主线程
		room.startRoomTask();

		// 正常返回
		response.setData(ProtoUtil.roomToMessage(room, player, null));
	}
}
