package com.incubator.game.handler;

import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.data.entity.PlayerInfoPO;
import com.incubator.game.player.Player;
import com.incubator.game.room.Room;
import com.incubator.game.room.RoomManager;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * 登录游戏服
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.LoginReq_VALUE)
public class LoginGameHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.LoginRes_VALUE);

		// 判断参数
		String token = request.getDataValue("token", String.class, "");
//		if (token.isEmpty()) {
//			logger.info("登录token为空");
//			response.setCode(ProtoCommon.Code.PARAMETER_ERR_VALUE);
//			return;
//		}
		// 根据token获取用户信息

		String username = request.getDataValue("username", String.class, "");

		// 创建player
		String player_id = UUID.randomUUID().toString();
		PlayerInfoPO playerInfoPO = new PlayerInfoPO();
		playerInfoPO.player_id = player_id;
		playerInfoPO.token = player_id;
		playerInfoPO.name = "玩家" + username;
		Player player = PlayerUtil.createPlayer(playerInfoPO);

		// 绑定session
		player.bind(session, 0L, player.getId());
		logger.info("玩家进入游戏 playerId={}", player.getId());

		// 登录数据
		player.onLogin();

		// 玩家是否在房间
		Room room = RoomManager.getInstance().getRoomByPlayerId(player.getId());

		// 正常返回
		Map<String, Object> map = new HashMap<>();
		map.put("playerInfo", ProtoUtil.playerInfoToMessage(player));
		map.put("roomId", room != null? room.data.roomId : 0);
		response.setData(map);
	}
}
