package com.incubator.game.handler.jdgd;

import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.player.Player;
import com.incubator.game.room.RoomManager;
import com.incubator.game.room.JDGDRoom;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

/**
 * 经典掼蛋 过请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.PlayNoCardsReq_VALUE)
public class JDGDPassHandler extends NetHandler {

    @Override
    public void onDate(Connection session, WSRequest request, WSResponse response) {
        response.setCmd(CommonProto.Cmd.PlayNoCardsRes_VALUE);

        Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
        if (player == null) {
            logger.info("创建房间失败，找不到玩家");
            response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
            response.setMessage("操作失败...");
            return;
        }

        // 判断玩家绑定房间
        if (player.data.roomId <= 0) {
            logger.info("准备失败，玩家不在房间");
            response.setCode(CommonProto.Code.PLAYER_ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家不在房间...");
            return;
        }

        // 玩家房间是否存在
        JDGDRoom room = (JDGDRoom) RoomManager.getInstance().getRoom(player.data.roomId);
        if (room == null) {
            logger.info("玩家房间已解散");
            response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
            response.setMessage("操作失败，玩家房间已解散...");
            return;
        }

        // 当前状态是否可操作
        if (room.data.state != 2 || player.data.state != 2) {
            logger.info("房间或玩家不在游戏中状态...");
            response.setCode(7);
            response.setMessage("操作失败，房间或玩家不在游戏中状态...");
            return;
        }

        // 当前可操作玩家是否自己
        if (!room.data.currentPlayer.getId().equals(player.getId())) {
            logger.info("当前不是自己出牌回合...");
            response.setCode(8);
            response.setMessage("操作失败，当前不是自己出牌回合...");
            return;
        }

        // 玩家过
        room.playerPass(player);

        // 正常返回
        response.setData(ProtoUtil.roomToMessage(room, player, room.data.currentPlayer));
    }
}
