package com.incubator.game.handler.jdgd;

import com.incubator.common.MessageHandler;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.NetHandler;
import com.incubator.core.net.ws.WSRequest;
import com.incubator.core.net.ws.WSResponse;
import com.incubator.game.room.JDGDRoom;
import com.incubator.game.player.Player;
import com.incubator.game.room.RoomManager;
import com.incubator.game.util.ProtoUtil;
import com.incubator.game.util.PlayerUtil;
import com.incubator.message.proto.CommonProto;

/**
 * 经典掼蛋 准备请求
 *
 * @author johnc
 */
@MessageHandler(id = CommonProto.Cmd.RoomReadyReq_VALUE)
public class JDGDReadyHandler extends NetHandler {

	@Override
	public void onDate(Connection session, WSRequest request, WSResponse response) {
		response.setCmd(CommonProto.Cmd.RoomReadyRes_VALUE);

		Player player = (Player) PlayerUtil.getOnlinePlayer(session.getPlayerId());
		if (player == null) {
			logger.info("创建房间失败，找不到玩家");
			response.setCode(CommonProto.Code.SYSTEM_ERR_VALUE);
			response.setMessage("操作失败...");
			return;
		}

		// 判断玩家绑定房间
		if (player.data.roomId <= 0) {
			logger.info("准备失败，玩家不在房间");
			response.setCode(CommonProto.Code.PLAYER_ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("操作失败，玩家不在房间...");
			return;
		}

		// 玩家房间是否存在
		JDGDRoom room = (JDGDRoom) RoomManager.getInstance().getRoom(player.data.roomId);
		if (room == null) {
			logger.info("玩家房间已解散");
			response.setCode(CommonProto.Code.ROOM_NOT_EXIST_ERR_VALUE);
			response.setMessage("操作失败，玩家房间已解散...");
			return;
		}

		// 准备
		room.playerReady(player);

		// 正常返回
		response.setData(ProtoUtil.roomToMessage(room, player, null));
	}
}
