package com.incubator.game.listener;

import com.incubator.common.net.Connection;
import com.incubator.core.net.handler.listener.GameHttpServerConnectionListener;
import com.incubator.core.net.http.HttpRequestHandler;
import com.incubator.game.GGame;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.Unpooled;
import io.netty.handler.codec.http.*;
import io.netty.util.CharsetUtil;
import io.netty.util.ReferenceCountUtil;

import java.nio.charset.StandardCharsets;

public class HttpInnerListener extends GameHttpServerConnectionListener {

	@Override
	public void messageArrived(Connection conn, HttpRequest msg) {
		// 因为此方法执行完后就会回收msg,所以这里保留一次
		ReferenceCountUtil.retain(msg);
		QueryStringDecoder decode = new QueryStringDecoder(msg.uri());
		//获取路径
		String path = decode.path();
		if ("/favicon.ico".equals(path)) {
			return;
		}
		//获取参数
		String body = "";
		try {
			if (msg.method() == HttpMethod.POST) {
				if (msg instanceof FullHttpRequest) {
					FullHttpRequest request = (FullHttpRequest) msg;
					body = this.getBody(request);
				}
			}

			com.incubator.core.net.http.HttpRequest request = new com.incubator.core.net.http.HttpRequest();
			HttpRequestHandler handler = GGame.httpHandlers.get(path);
			if (handler != null) {
				request.bindSession(conn);
				logger.info("收到http消息 : path={}", path);
				com.incubator.core.net.http.HttpResponse response = handler.onDate(request, body);
				if (response != null) {
					response.setPath(path);
					FullHttpResponse httpResponse = new DefaultFullHttpResponse(HttpVersion.HTTP_1_1, HttpResponseStatus.OK, Unpooled.wrappedBuffer(response.toJson().getBytes(StandardCharsets.UTF_8)));
					httpResponse.headers().set(HttpHeaderNames.CONTENT_TYPE, "text/plain; charset=UTF-8");
					conn.writeAndFlush(httpResponse);
					conn.close();
					logger.info("返回http消息 : path={}, 返回内容 : {}", path, response.toJson());
				}
			}
		} catch (Exception e) {
			logger.error("====http调用" + path + "出错：", e);
		} finally {
			ReferenceCountUtil.release(msg);
		}
	}

	@Override
	public void connectionOpened(Connection connection) {
		
	}

	@Override
	public void connectionClosed(Connection connection) {
		
	}

	/**
	 * 获取body参数
	 * @param request
	 * @return
	 */
	private String getBody(FullHttpRequest request){
		ByteBuf buf = request.content();
		return buf.toString(CharsetUtil.UTF_8);
	}
}
