package com.incubator.game.player;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.common.net.Connection;
import com.incubator.core.net.ws.Message;
import com.incubator.core.player.GPlayerBase;
import com.incubator.game.GGame;
import org.slf4j.Logger;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.Map;

/**
 * 功能描述：玩家的基类
 */
public abstract class GPlayer extends GPlayerBase {

	protected Logger logger = Log4jUtil.getLogger(getClass());

	/** 玩家绑定的链接 **/
	protected Connection session;
	/** 0首次进入场景 1非首次进入场景 **/
	protected byte state = 0;
	/** 用户id **/
	private long userId = 0;
	
	public GPlayer() {}

	/**
	 * 绑定session
	 *
	 * @param session
	 * @param userid 账号id
	 * @param playerId 玩家id
	 */
	public void bind(Connection session, long userid, String playerId) {
		synchronized (this) {			
			this.userId = userid;
			this.lastUpdateTime = GGame.APP_TIME;
			session.setUid(userid);
			session.setPlayerId(playerId);
			this.session = session;
		}
	}

	/** 获取玩家状态 */
	public byte getState() {
		return this.state;
	}

	public void setState(byte state) {
		this.state = state;
	}

	/**
	 * 功能描述：绑定的客户端会话
	 */
	public Connection getSession() {
		return this.session;
	}

	/** 功能描述：玩家接收广播消息 */
	public void receive(int cmd, Map<String, Object> data) {
		try {
			if (this.session != null) {
				Message msg = new Message();
				msg.setCmd(cmd);
				msg.setData(data);
				this.session.writeAndFlush(msg.toBytes());
				logger.info("推送消息 : cmd={}, 推送内容 : {}", msg.getCmd(), msg.toJson());
			}
		} catch (Exception e) {
			logger.error("玩家发送信息出错 : ", e);
		}
	}

	/** 功能描述：处理退出游戏世界 */
	public int doLogout() {
		synchronized (this) {
			int gameTimes = 0;
			long lStart = GGame.APP_TIME;
			
			this.setState((byte)0);//重置首次状态
			GGame.onlinePlayers.remove(this.getId());
			gameTimes = setLogoutTime(new Date());
			this.onLogoutUpdate();
			this.sync();

			String ip = this.session.getRemoteAddress() == null ? "" : this.session.getRemoteAddress().toString();
			logger.info(LocalDateTime.now().toLocalTime().toString() + " -- 清理绑定session : id = ",
					this.getId(), ", ip =" + ip, "， 耗时：", GGame.APP_TIME - lStart);
			
			return gameTimes;
		}	
	}
}
