package com.incubator.game.player;

import com.incubator.game.GGame;
import com.incubator.game.data.entity.PlayerInfoPO;

import java.util.Date;

/**
 * 玩家实体
 */
public class Player extends GPlayer {

    /** 玩家数据 **/
    public PlayerInfoPO data;

    public Player(PlayerInfoPO playerInfoPO) {
        try {
            this.data = playerInfoPO;
        } catch (Exception e) {
            logger.error("玩家初始化数据出错...{}", e.getMessage());
        }
    }

    /**
     * 角色登录成功, 服务器内部处理逻辑在此
     */
    public void onLogin() {
        this.logger.info("玩家登录成功...");
        //0点重置数据
//        Date now = new Date();
//        if (DateUtils.isSameDay(now, this.memePunksData.refreshTime)) {
//            return;
//        }
        this.refreshNewDay();

        // 加入在线玩家集合
        GGame.addPlayer(this);
    }

    public void reset() {
        this.data = new PlayerInfoPO();
    }

    /**
     * 销毁玩家，释放资源
     */
    public void destroy() {
        reset();
        logger.debug("Player {} has been destroyed.", this.data.player_id);
        this.doLogout();
    }

    /**
     * 每日刷新
     */
    public void refreshNewDay() {
        this.logger.info("每日刷新玩家数据...");
    }

    @Override
    public void onLogoutUpdate() {
        this.logger.info("退出更新, 退出清理...");
    }

    @Override
    public void sync() {
        this.logger.info("数据安全持久化...");
    }

    @Override
    public void free() {
        this.logger.info("释放玩家的相关引用...");
    }

    @Override
    public void playerRemoveSessionBind() {
        this.logger.info("玩家移除绑定了...");
    }

    @Override
    public void forbidPlayer(int forbidSec, String desc) {

    }

    @Override
    public int setLogoutTime(Date date) {
        return 0;
    }

    @Override
    public long getLogoutTime() {
        return 0;
    }

    @Override
    public void onPlayerUpdate() {

    }

    @Override
    public String getId() {
        return this.data.player_id;
    }

    @Override
    public int getLogicServerId() {
        return GGame.SERVER_ID;
    }

    @Override
    public long getUid() {
        return this.session.getUid();
    }

    @Override
    public void doUpdate() {

    }

}
