package com.incubator.game.room;

import com.incubator.game.player.Player;
import com.incubator.core.room.GRoomInterface;
import com.incubator.game.util.CardUtils;
import com.incubator.game.util.JDGDUtils;
import com.incubator.game.util.ProtoUtil;
import com.incubator.message.proto.CommonProto;

import java.util.*;

/**
 * 经典掼蛋房间
 */
public class JDGDRoom extends Room implements GRoomInterface {

    public JDGDRoom() {}

    @Override
    public void init(Player player, int type) {
        this.data.roomId = RoomManager.getInstance().randomRoomId();
        this.data.type = type;
        this.data.ownerId = player.getId();
        this.data.maxNum = 2;
        if (!this.data.playerIds.containsKey(player.getId())) {
            this.data.playerIds.put(player.getId(), player);
        }
        this.data.playerMap.put(0, player);
        // 初始化级牌点数
        this.data.curLevelPoint = JDGDUtils.levelPoint[this.data.curLevelIndex];
        this.initCardList();
    }

    /**
     * 初始化牌库
     */
    @Override
    public void initCardList() {
        for (int i = 0; i < 2; i++) {
            this.data.cardList.addAll(Arrays.asList(JDGDUtils.values));
            Collections.shuffle(this.data.cardList);
        }
    }

    /**
     * 主线程逻辑
     */
    @Override
    public void doUpdate() {
//        logger.debug("检测: 经典掼蛋房间主线程执行...");
        switch (this.data.state) {
            case 0:
                // 等待状态
                if (this.checkReadyStart()) {
                    this.data.state = 1;
                    this.data.flag = false;
                    this.data.time = 0;
                }

                // 120秒未开始直接解散
                if (this.data.time >= 120) {
                    logger.debug("解散房间...");
                    RoomManager.getInstance().releaseRoom(this);
                }
                break;
            case 1:
                // 开局
                if (!this.data.flag) {
                    //开始
                    if (this.data.time >= 3) {
                        this.doStart();
                        this.data.state = 2;
                        this.data.flag = false;
                        this.data.time = 0;
                        logger.debug("检测: 房间号:{}, 房间人数:{}, 当前局数:{}, 状态:开局...VS动画...", this.data.roomId, this.data.playerMap.size(), this.data.curRound);
                    }
                }
                break;
            case 2:
                // 进行中
                if (!this.data.flag) {
                    // 检查玩家进贡、还贡是否完成
                    if (this.checkTributeStart()) {
                        this.data.flag = true;
                        this.data.time = 0;
                    }

                    // 20秒强制操作进贡、还贡
                    if (this.data.time >= 20) {
                        this.doTribute();
                        this.data.flag = true;
                        this.data.time = 0;
                    }
                } else {
                    // 20秒未操作,强制过
//                    if (this.data.time >= 20) {
//                        logger.debug("强制操作过...");
//                        this.playerPass(this.data.currentPlayer);
//                        this.data.flag = true;
//                        this.data.time = 0;
//                    }
                }
                break;
            case 3:
                // 结束
                if (!this.data.flag) {
                    this.data.flag = true;
                    this.data.time = 0;
                    logger.debug("游戏结束: 房间号 : {}, 房间人数 : {}, 当前局数 : {}", this.data.roomId, this.data.playerMap.size(), this.data.curRound);
                } else {
                    // 120秒未开始直接解散
//                    if (this.data.time >= 120) {
//                        logger.debug("解散房间...");
//                        RoomManager.getInstance().releaseRoom(this);
//                    }
                }
                break;
        }

        this.data.time += 1;
    }

    /**
     * 玩家准备
     *
     * @param player
     */
    public void playerReady(Player player) {
        if (player.data.state == 1) {
            return;
        }

        // 设置玩家状态
        player.data.state = 1;

        // 推送房间其他3个玩家
        for (Player tmPlayer : this.data.playerMap.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = ProtoUtil.roomToMessage(this, tmPlayer, null);
                tmPlayer.receive(CommonProto.Cmd.ReadyStateUpdate_VALUE, data);
            }
        }
    }

    /**
     * 检测是否可以开始 3秒倒计时开始
     *
     * @return
     */
    @Override
    public boolean checkReadyStart() {
        if (!this.isStart() || this.data.state != 0) {
            return false;
        }

        // 广播3秒倒计时
        for (Player tmPlayer : this.data.playerMap.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = ProtoUtil.roomToMessage(this, tmPlayer, null);
                data.put("state", 1);
                data.put("time", 3);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }

        return true;
    }

    /**
     * 是否玩家都已准备,可开始
     *
     * @return
     */
    @Override
    public boolean isStart() {
        int cnt = 0;
        for (Player player : this.data.playerMap.values()) {
            if (player != null && player.data.state == 1) {
                cnt += 1;
            }
        }
        return cnt >= this.data.maxNum;
    }

    /**
     * 给玩家发牌，每人27张
     */
    @Override
    public void doSendCard() {
        for (Player player : this.data.playerMap.values()) {
            if (player != null) {
                // 设置用户状态
                player.data.state = 2;
                // 设置级牌点数
                if (this.data.curRound == 1) {
                    // 我方级数
                    player.data.levelPointView[0] = JDGDUtils.levelPoint[0];
                    // 对方级数
                    player.data.levelPointView[1] = JDGDUtils.levelPoint[0];
                }
                // 设置队友位置
                if (this.data.maxNum >= 4) {
                    Integer[] nextPos = this.getNextPos(player.data.pos);
                    Player tmpPlayer = this.data.playerMap.get(nextPos[1]);
                    player.data.teammateId = tmpPlayer != null ? tmpPlayer.data.teammateId : null;
                }
                // 发牌
                for (int i = 0; i < 27; i++) {
                    int card = this.data.cardList.get(0);
                    player.data.remainCards = CardUtils.add(player.data.remainCards, card);
                    this.data.cardList.remove(0);
                }
            }
        }
    }

    /**
     * 游戏开局
     */
    @Override
    public void doStart() {
        this.data.curRound += 1;
        // 定庄
        this.data.zhuangPos = 0;
        // 设置当前操作玩家
        this.data.currentPlayer = this.data.playerMap.get(this.data.zhuangPos);
        // 发牌
        this.doSendCard();
        // 广播
        for (Player tmPlayer : this.data.playerMap.values()) {
            if (tmPlayer != null) {
                Map<String, Object> data = ProtoUtil.roomToMessage(this, tmPlayer, this.data.currentPlayer);
                data.put("state", 2);
                data.put("time", 20);
//                CardUtils.printCards(tmPlayer.data.remainCards);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
    }

    /**
     * 开局检测是否进贡、还贡
     *
     * @return
     */
    public void checkTribute() {
        // 0.先判断人数
        switch (this.data.maxNum) {
            case 2:// 2人房
                // 判断单贡

                break;
            case 4:// 4人房
                // 1.判断单贡, 13游一队,24游一队

                // 2.判断双贡, 12游一队,34游一队
                break;
        }
    }

    /**
     * 检测进贡、还贡是否完成
     *
     * @return
     */
    public boolean checkTributeStart() {

        return false;
    }

    /**
     * 强制进贡、还贡
     */
    public void doTribute() {

    }

    /**
     * todo 根据当前玩家位置, 获取下一个玩家位置
     *
     * @param pos 当前操作玩家的位置
     * @return
     */
    public Integer[] getNextPos(int pos) {
        Integer[] nextpos = null;
//        switch (pos) {
//            case 3:
//                nextpos = new Integer[] { 0, 1, 2 };
//                break;
//            case 2:
//                nextpos = new Integer[] { 3, 0, 1 };
//                break;
//            case 1:
//                nextpos = new Integer[] { 2, 3, 0 };
//                break;
//            case 0:
//                nextpos = new Integer[] { 1, 2, 3 };
//                break;
//        }
        switch (pos) {
            case 1:
                nextpos = new Integer[] { 0 };
                break;
            case 0:
                nextpos = new Integer[] { 1 };
                break;
        }
        return nextpos;
    }

    /**
     * 根据当前玩家位置, 获取下一个可操作玩家
     *
     * @param player 当前操作玩家
     * @return
     */
    public Player getNextPlayer(Player player) {
        Player nextPlayer = null;
        Integer[] nextPos = this.getNextPos(player.data.pos);
        for (Integer index : nextPos) {
            Player temPlayer = this.data.playerMap.get(index);
            if (temPlayer.data.state == 2 && !temPlayer.getId().equals(player.getId())) {
                nextPlayer = temPlayer;
                break;
            }
        }
        return nextPlayer;
    }

    /**
     * 判断玩家出的牌是否可以压过上一手牌
     *
     * @param disCardsList 当前玩家出的牌
     * @param disCardsType 当前玩家出的牌型
     * @param currentLevel 当前级数
     * @return 是否可以出牌
     */
    public boolean canDisCard(int[] disCardsList, JDGDUtils.CardType disCardsType, int currentLevel) {
        // 0.桌上无牌直接出
        if (this.data.curDisCardList == null || this.data.curDisCardType == null) {
            return true;
        }

        // 1.牌型不一样,判断手牌是否是炸弹
        if (disCardsType != this.data.curDisCardType && !JDGDUtils.isBomb(disCardsType)) {
            return false;
        }

        // 获取权重值
        int previousWeight = JDGDUtils.getCardTypeWeight(this.data.curDisCardType);
        int currentWeight = JDGDUtils.getCardTypeWeight(disCardsType);

        // 权重不同，按权重大小比较
        if (currentWeight > previousWeight) {
            return true;
        }

        // 权重相同，根据牌点大小进行比较（仅适用于同类型的牌型，如炸弹或顺子）
        if (currentWeight == previousWeight) {
            return JDGDUtils.compareCardStrength(this.data.curDisCardList, disCardsList, currentLevel, disCardsType) > 0;
        }

        return false; // 其他情况不能出牌
    }

    /**
     * 玩家出牌
     *
     * @param player    出牌玩家
     * @param disCardList   打出的牌
     * @param disCardType   打出的牌型
     */
    public void playerDisCard(Player player, int[] disCardList, JDGDUtils.CardType disCardType) {
        // 是否自己回合
        if (!this.data.currentPlayer.getId().equals(player.getId())) {
            return;
        }
        if (CardUtils.contains(player.data.remainCards, disCardList)) {
            logger.info("出牌...玩家ID:{}, 位置:{}, 出牌牌型:{}, 出牌:{}, 手牌大小:{}, 手牌信息: {}",
                    player.getId(), player.data.pos, disCardType, Arrays.toString(disCardList),
                    player.data.remainCards.length, Arrays.toString(player.data.remainCards));
            // 从手牌中移除
            for (Integer card : disCardList) {
                player.data.remainCards = CardUtils.remove(player.data.remainCards, card);
            }
            // 设置出牌堆
            this.data.curDisCardList = disCardList;
            this.data.curDisCardType = disCardType;
            // 设置当前出牌的玩家
            this.data.disCardPlayer = player;
            // 出完牌设置结算数据
            if (player.data.remainCards.length == 0) {
                player.data.state = 3;
                this.data.result.put(this.data.result.size() + 1, player);
            }
            // 设置下一个操作玩家
            this.data.currentPlayer = this.getNextPlayer(player);
            // 广播其他3人
            for (Player tmPlayer : this.data.playerMap.values()) {
                if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                    Map<String, Object> data = ProtoUtil.roomToMessage(this, tmPlayer, this.data.currentPlayer);
                    tmPlayer.receive(CommonProto.Cmd.OtherPlayCards_VALUE, data);
                }
            }
            // 4人房间3人手牌出完，直接结算
            if (this.data.result.size() >= this.data.maxNum - 1) {
                // 结束
                this.gameOver();
            }
        }
    }

    /**
     * 玩家点击过
     *
     * @param player    点击过的玩家
     */
    public void playerPass(Player player) {
        // 是否自己回合
        if (!this.data.currentPlayer.getId().equals(player.getId())) {
            return;
        }
        logger.info("过...玩家ID:{}, 位置:{}, 手牌大小:{}, 手牌信息: {}", player.getId(), player.data.pos, player.data.remainCards.length, Arrays.toString(player.data.remainCards));
        // 添加过玩家列表
        this.data.passList.add(player.getId());
        // 4人房3人不出，清空出牌堆
        if (this.data.passList.size() >= this.data.maxNum - 1) {
            this.data.curDisCardList = new int[0];
            this.data.curDisCardType = null;
            this.data.passList.clear();
        }
        // 设置当前出牌玩家
        this.data.disCardPlayer = player;
        // 设置下一个操作玩家
        this.data.currentPlayer = this.getNextPlayer(player);
        // 广播其他3人
        for (Player tmPlayer : this.data.playerMap.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                Map<String, Object> data = ProtoUtil.roomToMessage(this, tmPlayer, this.data.currentPlayer);
                data.put("curDiscardList", new int[0]);
                tmPlayer.receive(CommonProto.Cmd.OtherPlayCards_VALUE, data);
            }
        }
    }

    /**
     * 升级
     */
    private void upgrade() {
        // 头游
        Player player = this.data.result.get(1);
        if (player == null) {
            return;
        }
        // 原级牌点数
        int oldLevelPoint = this.data.curLevelPoint;
        StringBuilder builder = null;
        switch (this.data.maxNum) {
            case 2:
                // 2人房
                builder = new StringBuilder();
                Player temPlayer = this.data.result.getOrDefault(1, null);
                if (temPlayer == null) {
                    return;
                }
                builder.append("头游玩家: ").append(player.data.name)
                        .append(" 升3级, ");
                // 设置当前级牌点数
                int index = Math.min(this.data.curLevelIndex + 3, JDGDUtils.levelPoint.length - 1);
                this.data.curLevelPoint = JDGDUtils.levelPoint[index];
                // 设置我方玩家级牌点数展示
                player.data.levelPointView[0] = this.data.curLevelPoint;
                // 设置对方级牌点数展示
                temPlayer.data.levelPointView[1] = this.data.curLevelPoint;
                break;
            case 4:
                // 4人房
                // 头游搭档
                for (Map.Entry<Integer, Player> entry : this.data.result.entrySet()) {
                    if (entry.getValue().getId().equals(player.data.teammateId)) {
                        builder = new StringBuilder();
                        switch (entry.getKey()) {
                            case 2:
                                // 搭档2游升3级
                                if (this.data.curLevelIndex + 3 >= JDGDUtils.levelPoint.length) {
                                    this.data.curLevelIndex = JDGDUtils.levelPoint.length;
                                } else {
                                    this.data.curLevelIndex += 3;
                                }
                                builder.append("头游玩家: ").append(player.data.name)
                                        .append(", 搭档玩家: ").append(entry.getValue().data.name)
                                        .append(" 2游升3级, ");
                                break;
                            case 3:
                                // 搭档3游升2级
                                if (this.data.curLevelIndex + 2 >= JDGDUtils.levelPoint.length) {
                                    this.data.curLevelIndex = JDGDUtils.levelPoint.length;
                                } else {
                                    this.data.curLevelIndex += 2;
                                }
                                builder.append("头游玩家: ").append(player.data.name)
                                        .append(", 搭档玩家: ").append(entry.getValue().data.name)
                                        .append(" 3游升2级, ");
                                break;
                            case 4:
                                // 搭档末游升1级
                                if (this.data.curLevelIndex + 1 >= JDGDUtils.levelPoint.length) {
                                    this.data.curLevelIndex = JDGDUtils.levelPoint.length;
                                } else {
                                    this.data.curLevelIndex += 1;
                                }
                                builder.append("头游玩家: ").append(player.data.name)
                                        .append(", 搭档玩家: ").append(entry.getValue().data.name)
                                        .append(" 末游升1级, ");
                                break;
                        }
                        // 设置当前级牌点数
                        int _index = Math.min(this.data.curLevelIndex + 3, JDGDUtils.levelPoint.length - 1);
                        this.data.curLevelPoint = JDGDUtils.levelPoint[_index];
                        // 设置我方玩家级牌点数展示
                        entry.getValue().data.levelPointView[0] = this.data.curLevelPoint;
                        player.data.levelPointView[0] = this.data.curLevelPoint;
                        // 设置对方级牌点数展示
                        for (Map.Entry<Integer, Player> entry1 : this.data.result.entrySet()) {
                            if (!entry1.getValue().getId().equals(player.getId())
                                    && !entry1.getValue().getId().equals(entry.getValue().getId())) {
                                entry1.getValue().data.levelPointView[1] = this.data.curLevelPoint;
                            }
                        }
                        break;
                    }
                }
                break;
        }
        logger.info("{}原级牌点数: {}, 当前级牌点数: {}", builder != null ? builder : "", oldLevelPoint, this.data.curLevelPoint);
    }

    /**
     * 牌局结算
     */
    private void settlement() {
        logger.debug("牌局结算...roomId : {}", this.data.roomId);
        // 结算信息
        Map<String, Object> settlementData = new HashMap<>();
        for (Map.Entry<Integer, Player> entry : this.data.result.entrySet()) {
            settlementData.put(entry.getKey().toString(), ProtoUtil.playerRoomToMessage(this, entry.getValue(), true));
        }
        // 广播结算
        for (Player tmPlayer : this.data.playerMap.values()) {
            if (tmPlayer != null) {
                // 结算信息
                Map<String, Object> data = ProtoUtil.roomToMessage(this, tmPlayer, this.data.currentPlayer);
                data.put("settlementInfo", settlementData);
                tmPlayer.receive(CommonProto.Cmd.GameStateChange_VALUE, data);
            }
        }
    }

    /**
     * 游戏结束
     */
    private void gameOver() {
        this.data.state = 3;
        this.data.time = 60;
        this.data.flag = false;

        List<String> list = new ArrayList<>();
        for (Player tmPlayer : this.data.result.values()) {
            if (tmPlayer != null) {
                list.add(tmPlayer.getId());
            }
        }

        for (Player player : this.data.playerMap.values()) {
            if (player != null) {
                player.data.state = 3;
                // 补充最后一个玩家
                if (!list.contains(player.getId())) {
                    this.data.result.put(this.data.result.size() + 1, player);
                }
            }
        }

        // 升级
        this.upgrade();
        // 结算
        this.settlement();
        logger.debug("牌局结束...roomId : {}", this.data.roomId);
    }
}
