package com.incubator.game.room;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.game.data.entity.RoomPO;
import com.incubator.game.player.Player;
import org.slf4j.Logger;

import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

/**
 * 房间实体类
 */
public class Room {

    protected Logger logger = Log4jUtil.getLogger(getClass());
    /** 激活标记 **/
    private boolean isActive;
    /** 房间任务主线程 **/
    private ScheduledExecutorService scheduler;
    /** 房间数据 **/
    public RoomPO data;

    public Room() {
        if (this.scheduler == null || this.scheduler.isShutdown()) {
            this.scheduler = Executors.newSingleThreadScheduledExecutor();
        }
        this.data = new RoomPO();
    }

    void init(Player player, int roomType) {}

    /**
     * 开启房间主线程任务
     */
    public void startRoomTask() {
        this.isActive = true;
        this.scheduler.scheduleAtFixedRate(() -> {
            if (this.isActive) {
                this.doUpdate();
            }
        }, 0, 1, TimeUnit.SECONDS);
    }

    /**
     * 房间主线程逻辑
     */
    void doUpdate() {}

    /**
     * 重置房间，用于对象池回收前清理状态
     */
    public void reset() {
        this.isActive = false;
    }

    /**
     * 销毁房间，释放资源 (对象池定期管理调用，不要手动调用)
     */
    public void destroy() {
        reset();
        this.scheduler.shutdownNow();
        this.scheduler = null;
        logger.debug("Room {} has been destroyed.", this.data.roomId);
        // 清空房间数据
        this.data = null;
    }
}
