package com.incubator.game.util;

import com.incubator.common.log4j.Log4jUtil;
import com.incubator.core.util.DataUtil;
import com.incubator.game.GGame;
import com.incubator.game.data.config.GlobalConfig;
import org.slf4j.Logger;

import java.io.File;
import java.io.FileFilter;
import java.util.Map;

import static com.incubator.game.GGame.env;

/**
 * 游戏配置管理类
 */
public final class GameDataUtil {

	private static final Logger logger;

	//private Map<String, Integer> IngroeSheet = new HashMap<>();
	private static int TableCount = 0;

	static {
		// 日志加载
		Log4jUtil.initLogConfig(env.getLogConfigFile());
		logger = Log4jUtil.getLogger(GameDataUtil.class);
	}

	private static void loadData(File parent) {
		File[] tables = parent.listFiles(new FileFilter() {
			@Override
			public boolean accept(File file) {
				if (file.isDirectory()) {
					loadData(file);
				}
				if (file.getName().endsWith(".json")) {
					DataUtil.loadData(file, false);
					return true;
				}
				return false;
			}
		});
		if (tables != null) {
			TableCount += tables.length;
		}
	}

	public static void initData() {
		DataUtil.addExtClass("com.incubator.game.data.data");
		File dataDir = new File(GGame.DIR_GAME_DATA);
		//读取策划表json文件
		loadData(dataDir);
		logger.info("读取策划表数量: {}", TableCount);
				
		DataUtil.initProperty();
		//加载全局参数
		GlobalConfig.init(true);
	}
	
	
	public static <T> T getConfigCo(Map<Integer, T> coMap) {
		return coMap.values().stream().findAny().get();
	}
}
