package com.incubator.game.util;

import com.incubator.game.player.Player;
import com.incubator.game.room.Room;

import java.util.*;

/**
 * 消息工具类
 */
public final class ProtoUtil {

    /**
     * 玩家信息转消息
     *
     * @param player
     * @return
     */
    public static Map<String, Object> playerInfoToMessage(Player player) {
        Map<String, Object> data = new HashMap<>();
        data.put("id", player.getId());
        data.put("name", player.data.name);
        data.put("avatar", "https://img95.699pic.com/element/40109/0194.png_300.png");
        data.put("sex", player.data.sex);
        data.put("exp", player.data.exp);
        data.put("level", player.data.level);
        data.put("vip", player.data.vip);
        return data;
    }

    /**
     * 房间信息转消息
     *
     * @param room
     * @param player 本人
     * @param opPlayer 当前操作玩家
     * @return
     */
    public static Map<String, Object> roomToMessage(Room room, Player player, Player opPlayer) {
        Map<String, Object> data = new HashMap<>();
        data.put("roomId", room.data.roomId);
        data.put("type", room.data.type);
        data.put("ownerId", room.data.ownerId);
        data.put("state", room.data.state);
        data.put("time", room.data.time);
        data.put("curRound", room.data.curRound);
        data.put("zhuangPos", room.data.zhuangPos);
        // 当前级牌点数
        data.put("curLevelPoint", room.data.curLevelPoint);
        data.put("opId", opPlayer != null ? opPlayer.getId() : "");
        data.put("opPos", opPlayer != null ? opPlayer.data.pos : -1);
        data.put("curDiscardList", room.data.curDisCardList);
        data.put("curDisCardType", room.data.curDisCardType);
        data.put("curDiscardPlayer", room.data.disCardPlayer != null ? room.data.disCardPlayer.getId() : "");
        data.put("curDiscardSex", room.data.disCardPlayer != null ? room.data.disCardPlayer.data.sex : 0);
        data.put("myInfo", playerRoomToMessage(room, player, true));
        List<Map<String, Object>> otherInfo = new ArrayList<>();
        for (Player tmPlayer : room.data.playerMap.values()) {
            if (tmPlayer != null && !Objects.equals(tmPlayer.getId(), player.getId())) {
                otherInfo.add(playerRoomToMessage(room, tmPlayer, false));
            }
        }
        data.put("otherInfo", otherInfo);
        return data;
    }

    /**
     * 玩家信息转消息
     *
     * @param room
     * @param player 玩家
     * @param flag 是否本人
     * @return
     */
    public static Map<String, Object> playerRoomToMessage(Room room, Player player, boolean flag) {
        Map<String, Object> data = playerInfoToMessage(player);
        data.put("diamond", player.data.diamond);
        data.put("pos", player.data.pos);
        data.put("state", player.data.state);
        data.put("isAuto", player.data.isAuto);
        // 级数显示 [我方级数, 对方级数]
        data.put("levelPointView", player.data.levelPointView);
        // 队友Id
        data.put("teammateId", player.data.teammateId);
        Map<String, Object> cardInfo = new HashMap<>();
        cardInfo.put("remainCards", flag? player.data.remainCards : "");
        cardInfo.put("remainCardsNum", player.data.remainCards.length);
        data.put("cardInfo", cardInfo);
        return data;
    }

}
